﻿/*
 * Datum: 25.09.2007
 * Zeit: 00:06
 * 
 */

using System;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Runtime.InteropServices;

namespace Booter.Telnet
{
	public struct MemoryStatus
	{
		internal uint dwLength;
		/// <summary>
		/// Specifies a number between 0 and 100 that gives a general idea of current memory utilization, in which 0 indicates no memory use and 100 indicates full memory use.
		/// </summary>
		public int MemoryLoad;
		/// <summary>
		/// Indicates the total number of bytes of physical memory.
		/// </summary>
		public int TotalPhysical;
		/// <summary>
		/// Indicates the number of bytes of physical memory available.
		/// </summary>
		public int AvailablePhysical;
		/// <summary>
		/// Indicates the total number of bytes that can be stored in the paging file. Note that this number does not represent the actual physical size of the paging file on disk.
		/// </summary>
		public int TotalPageFile;
		/// <summary>
		/// Indicates the number of bytes available in the paging file.
		/// </summary>
		public int AvailablePageFile;
		/// <summary>
		/// Indicates the total number of bytes that can be described in the user mode portion of the virtual address space of the calling process.
		/// </summary>
		public int TotalVirtual;
		/// <summary>
		/// Indicates the number of bytes of unreserved and uncommitted memory in the user mode portion of the virtual address space of the calling process.
		/// </summary>
		public int AvailableVirtual;
	}
	
	
	/// <summary>
	/// Description of TelnetSession.
	/// </summary>
	public class TelnetSession
	{
		private string _currentDir;
		Socket socket;
		public StreamReader Reader;
		public StreamWriter Writer;

		public TelnetSession(Socket socket)
		{
            Log.debug("new Telnet session from " + socket.RemoteEndPoint);
			_currentDir = @"\Hard Disk\";
			this.socket = socket;
			Reader = new StreamReader(new NetworkStream(socket, false));
			Writer = new StreamWriter(new NetworkStream(socket, true));
			new Thread(ClientLoop).Start();
		}

		void ClientLoop()
		{
			try
			{
				OnConnect();
				ShowPrompt();

				while (true)
				{
					Writer.Flush();

					string line = Reader.ReadLine();
					if (line == null)
					{
						break;
					}

					try
					{
						ProcessLine(line);
					}
					catch (Exception e)
					{
						Writer.WriteLine("error while processing command: "+e);
					}
					ShowPrompt();
				}
			}
			finally
			{
				socket.Close();
				OnDisconnect();
			}
		}

		void OnConnect()
		{
			Writer.WriteLine("X300T at your service.");
			Writer.WriteLine("WinCE Version:"+ System.Environment.OSVersion);
			Writer.WriteLine("DotNet Version:"+ System.Environment.Version);
		}

		void OnDisconnect()
		{
		}
		
		void ShowPrompt()
		{
			Writer.Write("# " + _currentDir + " >");
		}

		void ProcessLine(string line)
		{
			string[] parts = line.Split(new char[] {' '});
			switch (parts[0])
			{
				case "help":
					showHelp();
					break;
				case "ls":
					listDir(parts);
					break;
				case "mem":
					showMemory();
					break;
				case "reboot":
					Reboot();
					break;
				case "guid":
					showGUID();
					break;
				case "tv2":
					restartApp();
					break;
			}
		}
		
		void restartApp()
		{
            Writer.WriteLine("restarting tv2 application");
            int pid = App.StartTV2();
			
			if (pid != -1)
			{
				Writer.WriteLine("tv2 is now running with PID:" + pid);
			}
		}
		
		void showGUID()
		{
			byte[] buffer = new byte[0x18];
			GetBoxUniqueID(buffer, buffer.Length);
			string id = Encoding.ASCII.GetString(buffer, 0, 0x18);
			Writer.WriteLine("box id: "+id);
		}
		
		void showHelp()
		{
			Writer.WriteLine("known commands:");
			Writer.WriteLine("  help    this page");
			Writer.WriteLine("  ls      list files");
			Writer.WriteLine("  reboot  reboot system");
			Writer.WriteLine("  mem     shows memory status");
			Writer.WriteLine("  guid    shows unique box id");
			Writer.WriteLine("  tv2     restarts the tv2 application");
		}
		
		void listDir(string[] args)
		{
			Writer.WriteLine("not implemented yet.");
		}
		
		void showMemory()
		{
			MemoryStatus ms = new MemoryStatus();
			GlobalMemoryStatusCE( out ms );
			
			Writer.WriteLine("Physical mem available: {1}/{2}",ms.AvailablePhysical, ms.TotalPhysical);
			Writer.WriteLine("Virtual mem available: {1}/{2}",ms.AvailableVirtual, ms.TotalVirtual);
		}
		
		[DllImport("coredll", EntryPoint="GlobalMemoryStatus", SetLastError=false)]
		internal static extern void GlobalMemoryStatusCE(out MemoryStatus msce);
		
		[DllImport("booter.dll")]
		private static extern void Reboot();
		
		[return: MarshalAs(UnmanagedType.U1)]
		[DllImport("booter.dll")]
		private static extern bool GetBoxUniqueID(byte[] buffer, int length);
		
	}
}
